Value

Enum Value 

pub enum Value {
    Null,
    Boolean(bool),
    Number(NumberBuf),
    String(String),
    Array(Array),
    Object(Object),
}
JSON Value.

Parsing

You can parse a Value by importing the Parse trait providing a collection of parsing functions.

Example

use jstrict::{Value, Parse, CodeMap};
let (value, code_map) = Value::parse_str("{ \"key\": \"value\" }").unwrap();

The code_map value of type CodeMap contains code-mapping information about all the fragments of the JSON value (their location in the source text).

Comparison

This type implements the usual comparison traits PartialEq, Eq, PartialOrd and Ord. However by default JSON object entries ordering matters, meaning that { "a": 0, "b": 1 } is not equal to { "b": 1, "a": 0 }. If you want to do comparisons while ignoring entries ordering, you can use the Unordered type (combined with the UnorderedPartialEq trait). Any T reference can be turned into an Unordered<T> reference at will using the BorrowUnordered::as_unordered method.

Example

use jstrict::{json, Unordered, BorrowUnordered};

let a = json!({ "a": 0, "b": 1 });
let b = json!({ "b": 1, "a": 0 });

assert_ne!(a, b); // not equals entries are in a different order.
assert_eq!(a.as_unordered(), b.as_unordered()); // equals modulo entry order.
assert_eq!(Unordered(a), Unordered(b)); // equals modulo entry order.

Printing

The Print trait provide a highly configurable printing method.

Example

use jstrict::{Value, Parse, Print};

let value = Value::parse_str("[ 0, 1, { \"key\": \"value\" }, null ]").unwrap().0;

println!("{}", value.pretty_print()); // multi line, indent with 2 spaces
println!("{}", value.inline_print()); // single line, spaces
println!("{}", value.compact_print()); // single line, no spaces

let mut options = jstrict::print::Options::pretty();
options.indent = jstrict::print::Indent::Tabs(1);
println!("{}", value.print_with(options)); // multi line, indent with tabs

Variants



Null

null.



Boolean(bool)

Boolean true or false.



Number(NumberBuf)

Number.



String(String)

String.



Array(Array)

Array.



Object(Object)

Object.

Implementations

Source

impl Value

pub fn from_serde_json(value: Value) -> Self

Available on crate feature serde_json only.

Converts a serde_json::Value into a Value.

Example
// First we create a `serde_json` value.
let a = serde_json::json!({
  "foo": 1,
  "bar": [2, 3]
});

// We convert the `serde_json` value into a `jstrict` value.
let b = jstrict::Value::from_serde_json(a);

// We convert it back into a `serde_json` value.
let _ = jstrict::Value::into_serde_json(b);
pub fn into_serde_json(self) -> Value

Available on crate feature serde_json only.

Converts a Value into a serde_json::Value.

Example
// First we create a `serde_json` value.
let a = serde_json::json!({
  "foo": 1,
  "bar": [2, 3]
});

// We convert the `serde_json` value into a `jstrict` value.
let b = jstrict::Value::from_serde_json(a);

// We convert it back into a `serde_json` value.
let _ = jstrict::Value::into_serde_json(b);
Source

impl Value

pub fn from_sonic_rs(value: Value) -> Self

Available on crate feature sonic-rs only.

Converts a sonic_rs::Value into a Value.

Example
// First we create a `sonic_rs` value.
let a = sonic_rs::json!({
  "foo": 1,
  "bar": [2, 3]
});

// We convert the `sonic_rs` value into a `jstrict` value.
let b = jstrict::Value::from_sonic_rs(a);

// We convert it back into a `sonic_rs` value.
let _ = jstrict::Value::into_sonic_rs(b);
pub fn into_sonic_rs(self) -> Value

Available on crate feature sonic-rs only.

Converts a Value into a sonic_rs::Value.

Example
// First we create a `sonic_rs` value.
let a = sonic_rs::json!({
  "foo": 1,
  "bar": [2, 3]
});

// We convert the `sonic_rs` value into a `jstrict` value.
let b = jstrict::Value::from_sonic_rs(a);

// We convert it back into a `sonic_rs` value.
let _ = jstrict::Value::into_sonic_rs(b);
Source

impl Value

pub fn get_fragment(&self, index: usize) -> Result<FragmentRef<'_>, usize>

Returns the fragment at the given index in traversal order, where index 0 is self.

Fragment indices match CodeMap offsets: the fragment returned for index i is described by code_map[offset + i]. On failure, returns the number of fragments left to skip.

pub const fn kind(&self) -> Kind

Returns the Kind of this value.

pub fn is_kind(&self, kind: Kind) -> bool

Checks if this value is of the given kind.

pub const fn is_null(&self) -> bool

Checks if this value is null.

pub const fn is_boolean(&self) -> bool

Checks if this value is a boolean.

pub const fn is_number(&self) -> bool

Checks if this value is a number.

pub const fn is_string(&self) -> bool

Checks if this value is a string.

pub const fn is_array(&self) -> bool

Checks if this value is an array.

pub const fn is_object(&self) -> bool

Checks if this value is an object.

pub fn is_empty_array_or_object(&self) -> bool

Checks if the value is either an empty array or an empty object.

pub const fn as_boolean(&self) -> Option<bool>

Returns this value as a bool, if it is a boolean.

pub const fn as_boolean_mut(&mut self) -> Option<&mut bool>

Returns a mutable reference to the inner bool, if this is a boolean.

pub fn as_number(&self) -> Option<&Number>

Returns this value as a Number, if it is a number.

pub const fn as_number_mut(&mut self) -> Option<&mut NumberBuf>

Returns a mutable reference to the inner number buffer, if this is a number.

pub fn as_string(&self) -> Option<&str>

Returns this value as a &str, if it is a string.

pub fn as_str(&self) -> Option<&str>

Alias for as_string.

pub const fn as_string_mut(&mut self) -> Option<&mut String>

Returns a mutable reference to the inner string, if this is a string.

pub fn as_array(&self) -> Option<&[Self]>

Returns this value as a slice of values, if it is an array.

pub const fn as_array_mut(&mut self) -> Option<&mut Array>

Returns a mutable reference to the inner array, if this is an array.

pub fn force_as_array(&self) -> &[Self]

Return the given value as an array, even if it is not an array.

Returns the input value as is if it is already an array, or puts it in a slice with a single element if it is not.

pub const fn as_object(&self) -> Option<&Object>

Returns this value as an Object, if it is an object.

pub const fn as_object_mut(&mut self) -> Option<&mut Object>

Returns a mutable reference to the inner object, if this is an object.

pub fn into_boolean(self) -> Option<bool>

Consumes this value, returning the inner bool if it is a boolean.

pub fn into_number(self) -> Option<NumberBuf>

Consumes this value, returning the inner number if it is a number.

pub fn into_string(self) -> Option<String>

Consumes this value, returning the inner string if it is a string.

pub fn into_array(self) -> Option<Array>

Consumes this value, returning the inner array if it is an array.

pub fn into_object(self) -> Option<Object>

Consumes this value, returning the inner object if it is an object.

pub fn traverse(&self) -> Traverse<'_>

Returns an iterator over every fragment of this value, in depth-first order, paired with its index.

The index of each fragment is its offset in the CodeMap produced when the value was parsed.

pub fn count(&self, f: impl FnMut(usize, FragmentRef<'_>) -> bool) -> usize

Recursively count the number of values for which f returns true.

pub fn volume(&self) -> usize

Returns the volume of the value.

The volume is the sum of all values and recursively nested values included in self, including self (the volume is at least 1).

This is equivalent to value.traverse().filter(|(_, f)| f.is_value()).count().

pub const fn take(&mut self) -> Self

Move and return the value, leaves null in its place.

pub fn canonicalize_with(&mut self, buffer: &mut Buffer)

Available on crate feature canonicalize only.

Puts this JSON value in canonical form according to RFC 8785.

The given buffer is used to canonicalize the number values.

pub fn canonicalize(&mut self)

Available on crate feature canonicalize only.

Puts this JSON value in canonical form according to RFC 8785.

For predictable performance in tight loops, use Self::canonicalize_with with a caller-owned buffer.